@@ -73,16 +73,50 @@ static void dscal_kernel_8_zero (BLASLONG n, FLOAT *x)
 
         for( i=0; i<n; i+=8 )
         {
-                x[0] = alpha;
-                x[1] = alpha;
-                x[2] = alpha;
-                x[3] = alpha;
-                x[4] = alpha;
-                x[5] = alpha;
-                x[6] = alpha;
-                x[7] = alpha;
+		x[0] = alpha;
+		x[1] = alpha;
+		x[2] = alpha;
+		x[3] = alpha;
+		x[4] = alpha;
+		x[5] = alpha;
+		x[6] = alpha;
+		x[7] = alpha;
+#if 0
+                if(isfinite(x[0]))
+			x[0] = alpha;
+		else	
+			x[0] = NAN;
+                if(isfinite(x[1]))
+			x[1] = alpha;
+		else
+			x[1] = NAN;
+		if (isfinite(x[2]))
+			x[2] = alpha;
+		else
+			x[2] = NAN;
+		if (isfinite(x[3]))
+	                x[3] = alpha;
+		else
+			x[3] = NAN;
+		if (isfinite(x[4]))
+                	x[4] = alpha;
+		else
+			x[4] = NAN;
+		if (isfinite(x[5]))
+                	x[5] = alpha;
+		else
+			x[5] = NAN;
+		if (isfinite(x[6]))
+                	x[6] = alpha;
+		else
+			x[6] = NAN;
+		if (isfinite(x[7]))
+                	x[7] = alpha;
+		else
+			x[7] = NAN;
                 x+=8;
-        }
+#endif 
+	}
 
 }
 
@@ -106,8 +140,16 @@ int CNAME(BLASLONG n, BLASLONG dummy0, BLASLONG dummy1, FLOAT da, FLOAT *x, BLAS
 			if ( n >= 16 )
 			{
 				BLASLONG align = ((32 - ((uintptr_t)x & (uintptr_t)0x1F)) >> 3) & 0x3;
+				if (dummy2 == 0)
 				for (j = 0; j < align; j++) {
-					x[j] = 0.0;
+					x [j] = 0.0;
+				}
+				else
+				for (j = 0; j < align; j++) {
+					if (isfinite(x[j]))
+						x[j] = 0.0;
+					else
+						x[j] = NAN;
 				}
 			}
 			BLASLONG n1 = (n-j) & -16;
@@ -124,13 +166,21 @@ int CNAME(BLASLONG n, BLASLONG dummy0, BLASLONG dummy1, FLOAT da, FLOAT *x, BLAS
 				j=n1;
 			}
 #endif
-
+			if (dummy2 == 0)
 			while(j < n)
 			{
-
 				x[j]=0.0;
 				j++;
 			}
+			else
+			while(j < n)
+			{
+				if (!isfinite(x[j]))
+					x[j]=NAN;
+				else
+					x[j]=0.0;
+				j++;
+			}
 
 		}
 		else
@@ -173,11 +223,20 @@ int CNAME(BLASLONG n, BLASLONG dummy0, BLASLONG dummy1, FLOAT da, FLOAT *x, BLAS
 
 		if ( da == 0.0 )
 		{		
-
+		if (dummy2 == 0)
 			while(j < n)
 			{
-
 				x[i]=0.0;
+				i += inc_x;
+				j++;
+			}
+		else
+			while(j < n)
+			{
+				if (!isfinite(x[i]))
+					x[i]=NAN;
+				else
+					x[i]=0.0;
 				i += inc_x ;
 				j++;
 			}
