@@ -1,5 +1,6 @@
 /*********************************************************************/
 /* Copyright 2009, 2010 The University of Texas at Austin.           */
+/* Copyright 2023 The OpenBLAS Project.                              */
 /* All rights reserved.                                              */
 /*                                                                   */
 /* Redistribution and use in source and binary forms, with or        */
@@ -49,7 +50,9 @@
 static void init_parameter(void);
 
 gotoblas_t TABLE_NAME = {
-  DTB_DEFAULT_ENTRIES ,
+  DTB_DEFAULT_ENTRIES,
+
+  SWITCH_RATIO,
 
   GEMM_DEFAULT_OFFSET_A, GEMM_DEFAULT_OFFSET_B, GEMM_DEFAULT_ALIGN,
 
@@ -63,6 +66,7 @@ gotoblas_t TABLE_NAME = {
 #endif
 
   SBGEMM_ALIGN_K,
+  0, // need_amxtile_permission
 
   sbstobf16_kTS, sbdtobf16_kTS, sbf16tos_kTS, dbf16tod_kTS,
 
@@ -1062,31 +1066,123 @@ static void init_parameter(void) {
 }
 #else // (ARCH_MIPS64)
 #if (ARCH_LOONGARCH64)
+static int get_L3_size() {
+  int ret = 0, id = 0x14;
+  __asm__ volatile (
+    "cpucfg %[ret], %[id]"
+    : [ret]"=r"(ret)
+    : [id]"r"(id)
+    : "memory"
+  );
+  return ((ret & 0xffff) + 1) * pow(2, ((ret >> 16) & 0xff)) * pow(2, ((ret >> 24) & 0x7f)) / 1024 / 1024; // MB
+}
 static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
 #endif
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+
+#if defined(LA464)
+  int L3_size = get_L3_size();
+#ifdef SMP
+  if(blas_num_threads == 1){
+#endif
+    //single thread
+    if (L3_size == 32){ // 3C5000 and 3D5000
+      TABLE_NAME.sgemm_p = 256;
+      TABLE_NAME.sgemm_q = 384;
+      TABLE_NAME.sgemm_r = 8192;
+
+      TABLE_NAME.dgemm_p = 112;
+      TABLE_NAME.dgemm_q = 289;
+      TABLE_NAME.dgemm_r = 4096;
+
+      TABLE_NAME.cgemm_p = 128;
+      TABLE_NAME.cgemm_q = 256;
+      TABLE_NAME.cgemm_r = 4096;
+
+      TABLE_NAME.zgemm_p = 128;
+      TABLE_NAME.zgemm_q = 128;
+      TABLE_NAME.zgemm_r = 2048;
+    } else { // 3A5000 and 3C5000L
+      TABLE_NAME.sgemm_p = 256;
+      TABLE_NAME.sgemm_q = 384;
+      TABLE_NAME.sgemm_r = 4096;
+
+      TABLE_NAME.dgemm_p = 112;
+      TABLE_NAME.dgemm_q = 300;
+      TABLE_NAME.dgemm_r = 3024;
+
+      TABLE_NAME.cgemm_p = 128;
+      TABLE_NAME.cgemm_q = 256;
+      TABLE_NAME.cgemm_r = 2048;
+
+      TABLE_NAME.zgemm_p = 128;
+      TABLE_NAME.zgemm_q = 128;
+      TABLE_NAME.zgemm_r = 1024;
+    }
+#ifdef SMP
+  }else{
+    //multi thread
+    if (L3_size == 32){ // 3C5000 and 3D5000
+      TABLE_NAME.sgemm_p = 256;
+      TABLE_NAME.sgemm_q = 384;
+      TABLE_NAME.sgemm_r = 1024;
+
+      TABLE_NAME.dgemm_p = 112;
+      TABLE_NAME.dgemm_q = 289;
+      TABLE_NAME.dgemm_r = 342;
+
+      TABLE_NAME.cgemm_p = 128;
+      TABLE_NAME.cgemm_q = 256;
+      TABLE_NAME.cgemm_r = 512;
+
+      TABLE_NAME.zgemm_p = 128;
+      TABLE_NAME.zgemm_q = 128;
+      TABLE_NAME.zgemm_r = 512;
+    } else { // 3A5000 and 3C5000L
+      TABLE_NAME.sgemm_p = 256;
+      TABLE_NAME.sgemm_q = 384;
+      TABLE_NAME.sgemm_r = 2048;
+
+      TABLE_NAME.dgemm_p = 112;
+      TABLE_NAME.dgemm_q = 300;
+      TABLE_NAME.dgemm_r = 738;
+
+      TABLE_NAME.cgemm_p = 128;
+      TABLE_NAME.cgemm_q = 256;
+      TABLE_NAME.cgemm_r = 1024;
+
+      TABLE_NAME.zgemm_p = 128;
+      TABLE_NAME.zgemm_q = 128;
+      TABLE_NAME.zgemm_r = 1024;
+    }
+  }
+#endif
+#else
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
   TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
   TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
 
-#ifdef BUILD_BFLOAT16
-  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
-#endif
-  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
-  TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
-  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
-  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
-
-#ifdef BUILD_BFLOAT16
-  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
-#endif
   TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
   TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
   TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
   TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+  TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+#endif
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
 }
 #else // (ARCH_LOONGARCH64)
 #if (ARCH_POWER)
@@ -1148,6 +1244,36 @@ static void init_parameter(void) {
 }
 #else //ZARCH
 
+#if (ARCH_RISCV64)
+static void init_parameter(void) {
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+#endif
+  TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
+  TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
+  TABLE_NAME.cgemm_p = CGEMM_DEFAULT_P;
+  TABLE_NAME.zgemm_p = ZGEMM_DEFAULT_P;
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+#endif
+  TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
+  TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
+  TABLE_NAME.cgemm_r = CGEMM_DEFAULT_R;
+  TABLE_NAME.zgemm_r = ZGEMM_DEFAULT_R;
+
+
+#ifdef BUILD_BFLOAT16
+  TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+#endif
+  TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
+  TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
+  TABLE_NAME.cgemm_q = CGEMM_DEFAULT_Q;
+  TABLE_NAME.zgemm_q = ZGEMM_DEFAULT_Q;
+}
+#else //RISCV64
+
 #ifdef ARCH_X86
 static int get_l2_size_old(void){
   int i, eax, ebx, ecx, edx, cpuid_level;
@@ -1244,6 +1370,10 @@ static __inline__ int get_l2_size(void){
 
   int eax, ebx, ecx, edx, l2;
 
+  l2 = readenv_atoi("OPENBLAS_L2_SIZE");
+  if (l2 != 0)
+    return l2;
+
   cpuid(0x80000006, &eax, &ebx, &ecx, &edx);
 
   l2 = BITMASK(ecx, 16, 0xffff);
@@ -1806,6 +1936,12 @@ static void init_parameter(void) {
 
 #endif
 
+#ifdef SAPPHIRERAPIDS
+#if (BUILD_BFLOAT16 == 1)
+  TABLE_NAME.need_amxtile_permission = 1;
+#endif
+#endif
+
 #if BUILD_COMPLEX==1
 #ifdef CGEMM3M_DEFAULT_P
   TABLE_NAME.cgemm3m_p = CGEMM3M_DEFAULT_P;
@@ -1940,6 +2076,7 @@ static void init_parameter(void) {
 
 
 }
+#endif //RISCV64
 #endif //POWER
 #endif //ZARCH
 #endif //(ARCH_LOONGARCH64)
